#!/usr/bin/env python3

import os
import sys
import shutil
import hashlib
import uuid
from subprocess import Popen
import click
import yaml
from myppa import __version__
from myppa.utils import *
from myppa.package import Package
import sqlite3
from pprint import pformat

@click.group()
@click.version_option(__version__)
def cli():
    pass

@cli.command()
def clean():
    cwd = ensure_cwd()
    cache_dir = os.path.join(cwd, "cache")
    if not click.confirm("Erase cache/ directory?"):
        return
    for filename in os.listdir(cache_dir):
        if filename == ".placeholder":
            continue
        fullpath = os.path.join(cache_dir, filename)
        if os.path.isdir(fullpath):
            shutil.rmtree(fullpath)
        else:
            os.remove(fullpath)

@cli.command()
def list():
    conn = sqlite3.connect(get_packages_db())
    c = conn.cursor()
    print("Packages with fixed versions")
    for row in c.execute("SELECT name, version from package WHERE NOT version_is_computed ORDER BY name"):
        print(row[0], row[1])
    print("Packages with version computed upon build")
    for row in c.execute("SELECT name, version from package WHERE version_is_computed ORDER BY name"):
        print(row[0], row[1])
    conn.close()

@cli.command()
def update():
    specs = []
    for root, dirs, files in os.walk(get_specs_dir()):
        for filename in files:
            if filename.endswith('.yaml'):
                specs.append(os.path.join(root, filename))
    click.echo("Total {} spec files found".format(len(specs)))
    if os.path.exists(get_packages_db()):
        os.remove(get_packages_db())
    conn = sqlite3.connect(get_packages_db())
    c = conn.cursor()
    c.execute("CREATE TABLE package (name TEXT, version_is_computed INTEGER, version TEXT, description TEXT)")
    for spec in specs:
        click.echo("Processing '{}'".format(spec))
        with open(spec, 'r') as f:
            for document in yaml.load_all(f):
                for name, package in document.items():
                    if not name.startswith("package-"):
                        continue
                    pkg = Package(package)
                    pkg.validate()
                    pkg.persist(conn)
    conn.commit()
    conn.close()

@cli.command()
@click.argument("package")
def show(package):
    click.echo(pformat(get_package_description(package), indent=2))

@cli.command()
@click.argument("package")
@click.argument("distribution", type=click.Choice(supported_distributions()))
@click.argument("architecture", type=click.Choice(supported_architectures()))
def script(package, distribution, architecture):
    click.echo(get_script(package, distribution, architecture))

@cli.command()
@click.argument("package")
@click.argument("distribution", type=click.Choice(supported_distributions()))
@click.argument("architecture", type=click.Choice(supported_architectures()))
def build(package, distribution, architecture):
    dist, codename = parse_distribution(distribution)
    script = get_script(package, distribution, architecture)
    scriptid = hashlib.sha1(script.encode('utf-8')).hexdigest()
    script_fullpath = os.path.join(get_cache_dir(), "{}.sh".format(scriptid))
    open(script_fullpath, 'w').write(script)
    tasks_dir = ensure_tasks_dir()
    taskid = str(uuid.uuid4())
    work_dir = os.path.join(tasks_dir, taskid)
    os.mkdir(work_dir)
    Popen(['sh', script_fullpath], cwd=work_dir).wait()
    outdir = "packages"
    for filename in os.listdir(work_dir):
        if filename.endswith(".deb"):
            shutil.copy(os.path.join(work_dir, filename), os.path.join(outdir, filename))

def main():
    return cli()

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python3

import os
import sys
import sqlite3
import click
from copy import copy
from jinja2 import Template
from myppa.package import Package

def supported_architectures():
    return ['i386', 'amd64']

def supported_distributions():
    supported_ubuntus = [
        "precise", "12.04",
        "trusty", "14.04",
        "xenial", "16.04",
    ]
    return ["ubuntu:" + i for i in supported_ubuntus]

def _normalize_codename(name):
    return {
        "804": "hardy",
        "1004": "lucid",
        "1104": "natty",
        "1110": "oneiric",
        "1204": "precise",
        "1210": "quantal",
        "1304": "raring",
        "1310": "saucy",
        "1404": "trusty",
        "1410": "utopic",
        "1504": "vivid",
        "1510": "wily",
        "1604": "xenial",
        "1610": "yakkety",
        "1704": "zesty",
    }.get(name.replace(".", ""), name)

def _default_codename(dist):
    return {
        "ubuntu": "xenial",
    }.get(dist)

def parse_package(package):
    nameversion = package.split("@")
    name = nameversion[0]
    version = nameversion[1] if len(nameversion) > 1 else None
    return name, version

def parse_distribution(distribution):
    distcodename = distribution.split(":")
    dist = distcodename[0]
    codename = distcodename[1] if len(distcodename) > 1 else _default_codename(dist)
    codename = _normalize_codename(codename)
    return dist, codename

def ensure_cwd():
    cwd = os.getcwd()
    required_dirs = ["packages", "cache", "specs"]
    required_files = ["myppa"]
    for directory in required_dirs:
        fulldir = os.path.join(cwd, directory)
        if not os.path.exists(fulldir) or not os.path.isdir(fulldir):
            raise RuntimeError("Directory \"{}\" is missing. Please cd to myppa root".format(directory))
    for filename in required_files:
        if not os.path.exists(os.path.join(cwd, filename)):
            raise RuntimeError("File \"{}\" is missing. Please cd to myppa root".format(filename))
    return cwd

def get_data_dir():
    root = os.path.abspath(os.path.dirname(__file__))
    return os.path.join(root, "data")

def get_data(*paths):
    return os.path.join(get_data_dir(), *paths)

def get_cache_dir():
    return os.path.join(ensure_cwd(), "cache")

def ensure_tasks_dir():
    tasks_dir = os.path.join(get_cache_dir(), "tasks")
    if not os.path.exists(tasks_dir):
        os.mkdir(tasks_dir)
    if not os.path.isdir(tasks_dir):
        raise RuntimeError("Cache directory is broken, run ./myppa clean and try again.")
    return tasks_dir

def get_packages_db():
    return os.path.join(ensure_cwd(), "cache", "packages.db")

def get_specs_dir():
    return os.path.join(ensure_cwd(), "specs")

def get_package_description(package):
    name, version = parse_package(package)
    conn = sqlite3.connect(get_packages_db())
    try:
        package = Package.load(conn, name, version)
    except RuntimeError as err:
        click.echo(err)
        sys.exit(1)
    conn.close()
    return package.description()

def get_script(package, distribution, architecture):
    description = get_package_description(package)
    env = copy(description)
    for k, v in description.items():
        env[k.replace("-", "_")] = v
    distribution, codename = parse_distribution(distribution)
    env['distribution'] = distribution
    env['codename'] = codename
    env['architecture'] = architecture
    template = Template(open(get_data("templates", "build_deb.sh"), 'r').read())
    return template.render(env)



from setuptools import setup

setup(
    name="myppa",
    setup_requires=["vcversioner"],
    vcversioner={
        'version_module_paths': ['myppa/_version.py'],
    },
    author="Stanislav Ivochkin",
    author_email="isn@extrn.org",
    description=("Manage your own PPA"),
    license="MIT",
    url="https://github.com/ivochkin/myppa-tool",
    packages=["myppa"],
    include_package_data=True,
    package_data={"myppa": ["data/templates/*"]},
    entry_points={
        "console_scripts": [
            "myppa = myppa.cli:main"
        ]
    },
    install_requires=[
        "click>=6.0",
        "Jinja2>=2.7",
        "PyYAML>=3.12",
    ],
)



#!/usr/bin/env python3

import subprocess
import json

class Package(object):
    def __init__(self, description):
        self._description = description

    def __getattr__(self, attr):
        return self._description.get(attr)

    def is_version_computed(self):
        return getattr(self, "version-script") is not None

    def validate(self):
        pass

    def description(self):
        return self._description

    def persist(self, conn):
        c = conn.cursor()
        c.execute("INSERT INTO package VALUES (?,?,?,?)",
            (getattr(self, "name"),
             self.is_version_computed(),
             getattr(self, "version"),
             json.dumps(self.description())))

    @classmethod
    def load(cls, conn, name, version):
        c = conn.cursor()
        description = None
        query = "SELECT description FROM package where name = ?"
        qargs = (name,)
        if version is not None:
            query += " AND version = ?"
            qargs = (name, version)
        row_found = False
        for row in c.execute(query, qargs):
            if row_found:
                raise RuntimeError(("Multiple packages with name '{}' are found. "
                                    "Please specify exact version of the package."
                                    ).format(name))
            row_found = True
            description = row[0]
        if not row_found:
            raise RuntimeError("No package with name '{}' and version '{}' is found."
                                .format(name, version or "any"))
            return 0
        return cls(json.loads(description))
